import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _defineProperty from "@babel/runtime/helpers/defineProperty";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import React from 'react';
import createElement from './create-element';
import checkForListedLanguage from './checkForListedLanguage';
var newLineRegex = /\n/g;

function getNewLines(str) {
  return str.match(newLineRegex);
}

function defaultLineNumberRenderer(_ref) {
  var number = _ref.number,
      style = _ref.style,
      showInlineLineNumbers = _ref.showInlineLineNumbers;
  return /*#__PURE__*/React.createElement("span", {
    key: "line-".concat(number),
    className: showInlineLineNumbers ? 'comment linenumber react-syntax-highlighter-line-number' : 'react-syntax-highlighter-line-number',
    style: style
  }, showInlineLineNumbers ? "".concat(number) : "".concat(number, "\n"));
}

function getAllLineNumbers(_ref2) {
  var lines = _ref2.lines,
      startingLineNumber = _ref2.startingLineNumber,
      style = _ref2.style,
      renderer = _ref2.renderer;
  return lines.map(function (_, i) {
    var number = i + startingLineNumber;
    var lineNumberRenderer = renderer || defaultLineNumberRenderer;
    return lineNumberRenderer({
      number: number,
      style: typeof style === 'function' ? style(number) : style,
      showInlineLineNumbers: false
    });
  });
}

function AllLineNumbers(_ref3) {
  var codeString = _ref3.codeString,
      codeStyle = _ref3.codeStyle,
      _ref3$containerStyle = _ref3.containerStyle,
      containerStyle = _ref3$containerStyle === void 0 ? {
    "float": 'left',
    paddingRight: '10px'
  } : _ref3$containerStyle,
      _ref3$numberStyle = _ref3.numberStyle,
      numberStyle = _ref3$numberStyle === void 0 ? {} : _ref3$numberStyle,
      startingLineNumber = _ref3.startingLineNumber,
      renderer = _ref3.renderer;
  return /*#__PURE__*/React.createElement("code", {
    style: Object.assign({}, codeStyle, containerStyle)
  }, getAllLineNumbers({
    lines: codeString.replace(/\n$/, '').split('\n'),
    style: numberStyle,
    startingLineNumber: startingLineNumber,
    renderer: renderer
  }));
}

function getEmWidthOfNumber(num) {
  return "".concat(num.toString().length, ".25em");
}

function getInlineLineNumber(lineNumber, inlineLineNumberStyle, renderer) {
  var lineNumberRenderer = renderer || defaultLineNumberRenderer;
  return lineNumberRenderer({
    number: lineNumber,
    style: inlineLineNumberStyle,
    showInlineLineNumbers: true
  });
}

function assembleLineNumberStyles(lineNumberStyle, lineNumber, largestLineNumber) {
  // minimally necessary styling for line numbers
  var defaultLineNumberStyle = {
    display: 'inline-block',
    minWidth: getEmWidthOfNumber(largestLineNumber),
    paddingRight: '1em',
    textAlign: 'right',
    userSelect: 'none'
  }; // prep custom styling

  var customLineNumberStyle = typeof lineNumberStyle === 'function' ? lineNumberStyle(lineNumber) : lineNumberStyle; // combine

  var assembledStyle = _objectSpread(_objectSpread({}, defaultLineNumberStyle), customLineNumberStyle);

  return assembledStyle;
}

function createLineElement(_ref4) {
  var children = _ref4.children,
      lineNumber = _ref4.lineNumber,
      lineNumberStyle = _ref4.lineNumberStyle,
      largestLineNumber = _ref4.largestLineNumber,
      showInlineLineNumbers = _ref4.showInlineLineNumbers,
      _ref4$lineProps = _ref4.lineProps,
      lineProps = _ref4$lineProps === void 0 ? {} : _ref4$lineProps,
      _ref4$className = _ref4.className,
      className = _ref4$className === void 0 ? [] : _ref4$className,
      showLineNumbers = _ref4.showLineNumbers,
      wrapLongLines = _ref4.wrapLongLines,
      lineNumberRenderer = _ref4.lineNumberRenderer;
  var properties = typeof lineProps === 'function' ? lineProps(lineNumber) : lineProps;
  properties['className'] = className;

  if (lineNumber && showInlineLineNumbers) {
    var inlineLineNumberStyle = assembleLineNumberStyles(lineNumberStyle, lineNumber, largestLineNumber);
    children.unshift(getInlineLineNumber(lineNumber, inlineLineNumberStyle, lineNumberRenderer));
  }

  if (wrapLongLines & showLineNumbers) {
    properties.style = _objectSpread(_objectSpread({}, properties.style), {}, {
      display: 'flex'
    });
  }

  return {
    type: 'element',
    tagName: 'span',
    properties: properties,
    children: children
  };
}

function flattenCodeTree(tree) {
  var className = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var newTree = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  var lineNumberRenderer = arguments.length > 3 ? arguments[3] : undefined;

  for (var i = 0; i < tree.length; i++) {
    var node = tree[i];

    if (node.type === 'text') {
      newTree.push(createLineElement({
        children: [node],
        className: _toConsumableArray(new Set(className)),
        lineNumberRenderer: lineNumberRenderer
      }));
    } else if (node.children) {
      var classNames = className.concat(node.properties.className);
      newTree = newTree.concat(flattenCodeTree(node.children, classNames, [], lineNumberRenderer));
    }
  }

  return newTree;
}

function processLines(codeTree, wrapLines, lineProps, showLineNumbers, showInlineLineNumbers, startingLineNumber, largestLineNumber, lineNumberStyle, wrapLongLines, lineNumberRenderer) {
  var _ref5;

  var tree = flattenCodeTree(codeTree.value);
  var newTree = [];
  var lastLineBreakIndex = -1;
  var index = 0;

  function createWrappedLine(children, lineNumber) {
    var className = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
    return createLineElement({
      children: children,
      lineNumber: lineNumber,
      lineNumberStyle: lineNumberStyle,
      largestLineNumber: largestLineNumber,
      showInlineLineNumbers: showInlineLineNumbers,
      lineProps: lineProps,
      className: className,
      showLineNumbers: showLineNumbers,
      wrapLongLines: wrapLongLines,
      lineNumberRenderer: lineNumberRenderer
    });
  }

  function createUnwrappedLine(children, lineNumber) {
    if (showLineNumbers && lineNumber && showInlineLineNumbers) {
      var inlineLineNumberStyle = assembleLineNumberStyles(lineNumberStyle, lineNumber, largestLineNumber);
      children.unshift(getInlineLineNumber(lineNumber, inlineLineNumberStyle, lineNumberRenderer));
    }

    return children;
  }

  function createLine(children, lineNumber) {
    var className = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
    return wrapLines || className.length > 0 ? createWrappedLine(children, lineNumber, className) : createUnwrappedLine(children, lineNumber);
  }

  var _loop = function _loop() {
    var node = tree[index];
    var value = node.children[0].value;
    var newLines = getNewLines(value);

    if (newLines) {
      var splitValue = value.split('\n');
      splitValue.forEach(function (text, i) {
        var lineNumber = showLineNumbers && newTree.length + startingLineNumber;
        var newChild = {
          type: 'text',
          value: "".concat(text, "\n")
        }; // if it's the first line

        if (i === 0) {
          var _children = tree.slice(lastLineBreakIndex + 1, index).concat(createLineElement({
            children: [newChild],
            className: node.properties.className,
            lineNumberRenderer: lineNumberRenderer
          }));

          var _line = createLine(_children, lineNumber);

          newTree.push(_line); // if it's the last line
        } else if (i === splitValue.length - 1) {
          var stringChild = tree[index + 1] && tree[index + 1].children && tree[index + 1].children[0];

          if (stringChild) {
            var lastLineInPreviousSpan = {
              type: 'text',
              value: "".concat(text)
            };
            var newElem = createLineElement({
              children: [lastLineInPreviousSpan],
              className: node.properties.className,
              lineNumberRenderer: lineNumberRenderer
            });
            tree.splice(index + 1, 0, newElem);
          } else {
            var _children2 = [newChild];

            var _line2 = createLine(_children2, lineNumber, node.properties.className);

            newTree.push(_line2);
          } // if it's neither the first nor the last line

        } else {
          var _children3 = [newChild];

          var _line3 = createLine(_children3, lineNumber, node.properties.className);

          newTree.push(_line3);
        }
      });
      lastLineBreakIndex = index;
    }

    index++;
  };

  while (index < tree.length) {
    _loop();
  }

  if (lastLineBreakIndex !== tree.length - 1) {
    var children = tree.slice(lastLineBreakIndex + 1, tree.length);

    if (children && children.length) {
      var lineNumber = showLineNumbers && newTree.length + startingLineNumber;
      var line = createLine(children, lineNumber);
      newTree.push(line);
    }
  }

  return wrapLines ? newTree : (_ref5 = []).concat.apply(_ref5, newTree);
}

function defaultRenderer(_ref6) {
  var rows = _ref6.rows,
      stylesheet = _ref6.stylesheet,
      useInlineStyles = _ref6.useInlineStyles;
  return rows.map(function (node, i) {
    return createElement({
      node: node,
      stylesheet: stylesheet,
      useInlineStyles: useInlineStyles,
      key: "code-segement".concat(i)
    });
  });
} // only highlight.js has the highlightAuto method


function isHighlightJs(astGenerator) {
  return astGenerator && typeof astGenerator.highlightAuto !== 'undefined';
}

function getCodeTree(_ref7) {
  var astGenerator = _ref7.astGenerator,
      language = _ref7.language,
      code = _ref7.code,
      defaultCodeValue = _ref7.defaultCodeValue;

  // figure out whether we're using lowlight/highlight or refractor/prism
  // then attempt highlighting accordingly
  // lowlight/highlight?
  if (isHighlightJs(astGenerator)) {
    var hasLanguage = checkForListedLanguage(astGenerator, language);

    if (language === 'text') {
      return {
        value: defaultCodeValue,
        language: 'text'
      };
    } else if (hasLanguage) {
      return astGenerator.highlight(language, code);
    } else {
      return astGenerator.highlightAuto(code);
    }
  } // must be refractor/prism, then


  try {
    return language && language !== 'text' ? {
      value: astGenerator.highlight(code, language)
    } : {
      value: defaultCodeValue
    };
  } catch (e) {
    return {
      value: defaultCodeValue
    };
  }
}

export default function (defaultAstGenerator, defaultStyle) {
  return function SyntaxHighlighter(_ref8) {
    var language = _ref8.language,
        children = _ref8.children,
        _ref8$style = _ref8.style,
        style = _ref8$style === void 0 ? defaultStyle : _ref8$style,
        _ref8$customStyle = _ref8.customStyle,
        customStyle = _ref8$customStyle === void 0 ? {} : _ref8$customStyle,
        _ref8$codeTagProps = _ref8.codeTagProps,
        codeTagProps = _ref8$codeTagProps === void 0 ? {
      className: language ? "language-".concat(language) : undefined,
      style: _objectSpread(_objectSpread({}, style['code[class*="language-"]']), style["code[class*=\"language-".concat(language, "\"]")])
    } : _ref8$codeTagProps,
        _ref8$useInlineStyles = _ref8.useInlineStyles,
        useInlineStyles = _ref8$useInlineStyles === void 0 ? true : _ref8$useInlineStyles,
        _ref8$showLineNumbers = _ref8.showLineNumbers,
        showLineNumbers = _ref8$showLineNumbers === void 0 ? false : _ref8$showLineNumbers,
        _ref8$showInlineLineN = _ref8.showInlineLineNumbers,
        showInlineLineNumbers = _ref8$showInlineLineN === void 0 ? true : _ref8$showInlineLineN,
        _ref8$startingLineNum = _ref8.startingLineNumber,
        startingLineNumber = _ref8$startingLineNum === void 0 ? 1 : _ref8$startingLineNum,
        lineNumberContainerStyle = _ref8.lineNumberContainerStyle,
        _ref8$lineNumberStyle = _ref8.lineNumberStyle,
        lineNumberStyle = _ref8$lineNumberStyle === void 0 ? {} : _ref8$lineNumberStyle,
        _ref8$lineNumberRende = _ref8.lineNumberRenderer,
        lineNumberRenderer = _ref8$lineNumberRende === void 0 ? defaultLineNumberRenderer : _ref8$lineNumberRende,
        wrapLines = _ref8.wrapLines,
        _ref8$wrapLongLines = _ref8.wrapLongLines,
        wrapLongLines = _ref8$wrapLongLines === void 0 ? false : _ref8$wrapLongLines,
        _ref8$lineProps = _ref8.lineProps,
        lineProps = _ref8$lineProps === void 0 ? {} : _ref8$lineProps,
        renderer = _ref8.renderer,
        _ref8$PreTag = _ref8.PreTag,
        PreTag = _ref8$PreTag === void 0 ? 'pre' : _ref8$PreTag,
        _ref8$CodeTag = _ref8.CodeTag,
        CodeTag = _ref8$CodeTag === void 0 ? 'code' : _ref8$CodeTag,
        _ref8$code = _ref8.code,
        code = _ref8$code === void 0 ? Array.isArray(children) ? children[0] : children : _ref8$code,
        astGenerator = _ref8.astGenerator,
        rest = _objectWithoutProperties(_ref8, ["language", "children", "style", "customStyle", "codeTagProps", "useInlineStyles", "showLineNumbers", "showInlineLineNumbers", "startingLineNumber", "lineNumberContainerStyle", "lineNumberStyle", "lineNumberRenderer", "wrapLines", "wrapLongLines", "lineProps", "renderer", "PreTag", "CodeTag", "code", "astGenerator"]);

    astGenerator = astGenerator || defaultAstGenerator;
    var allLineNumbers = showLineNumbers ? /*#__PURE__*/React.createElement(AllLineNumbers, {
      containerStyle: lineNumberContainerStyle,
      codeStyle: codeTagProps.style || {},
      numberStyle: lineNumberStyle,
      startingLineNumber: startingLineNumber,
      codeString: code,
      renderer: lineNumberRenderer
    }) : null;
    var defaultPreStyle = style.hljs || style['pre[class*="language-"]'] || {
      backgroundColor: '#fff'
    };
    var generatorClassName = isHighlightJs(astGenerator) ? 'hljs' : 'prismjs';
    var preProps = useInlineStyles ? Object.assign({}, rest, {
      style: Object.assign({}, defaultPreStyle, customStyle)
    }) : Object.assign({}, rest, {
      className: rest.className ? "".concat(generatorClassName, " ").concat(rest.className) : generatorClassName,
      style: Object.assign({}, customStyle)
    });

    if (!astGenerator) {
      return /*#__PURE__*/React.createElement(PreTag, preProps, allLineNumbers, /*#__PURE__*/React.createElement(CodeTag, codeTagProps, code));
    }
    /*
     * Some custom renderers rely on individual row elements so we need to turn wrapLines on
     * if renderer is provided and wrapLines is undefined.
     */


    if (wrapLines === undefined && renderer || wrapLongLines) wrapLines = true;
    renderer = renderer || defaultRenderer;
    var defaultCodeValue = [{
      type: 'text',
      value: code
    }];
    var codeTree = getCodeTree({
      astGenerator: astGenerator,
      language: language,
      code: code,
      defaultCodeValue: defaultCodeValue
    });

    if (codeTree.language === null) {
      codeTree.value = defaultCodeValue;
    } // determine largest line number so that we can force minWidth on all linenumber elements


    var largestLineNumber = codeTree.value.length + startingLineNumber;
    var rows = processLines(codeTree, wrapLines, lineProps, showLineNumbers, showInlineLineNumbers, startingLineNumber, largestLineNumber, lineNumberStyle, wrapLongLines, lineNumberRenderer);

    if (wrapLongLines) {
      codeTagProps.style = _objectSpread(_objectSpread({}, codeTagProps.style), {}, {
        whiteSpace: 'pre-wrap'
      });
    } else {
      codeTagProps.style = _objectSpread(_objectSpread({}, codeTagProps.style), {}, {
        whiteSpace: 'pre'
      });
    }

    return /*#__PURE__*/React.createElement(PreTag, preProps, /*#__PURE__*/React.createElement(CodeTag, codeTagProps, !showInlineLineNumbers && allLineNumbers, renderer({
      rows: rows,
      stylesheet: style,
      useInlineStyles: useInlineStyles
    })));
  };
}